create table resource_bundle_entries
( locale varchar2(2)
, resource_key varchar2(100)
, resource_value varchar2(100)
, context1 varchar2(100)
)

create or replace 
package nls_resource_manager
is

function get_resource_bundle_entries
( p_locale in varchar2
, p_context1 in varchar2
) return sys_refcursor
;

procedure update_resource_bundle_entry
( p_locale in varchar2
, p_context1 in varchar2 default null
, p_resource_key in varchar2
, p_resource_value in varchar2
);


end nls_resource_manager;


create or replace 
package body nls_resource_manager
is

function get_resource_bundle_entries
( p_locale in varchar2
, p_context1 in varchar2
) return sys_refcursor
is
  l_rc sys_refcursor;
begin
  open l_rc for 
    select resource_key
    ,      resource_value
    from   resource_bundle_entries
    where  locale= p_locale
    and   ( (p_context1 is null and context1 is null) 
          or
            p_context1 = context1
          )
    ;
    return l_rc;    
end get_resource_bundle_entries;

procedure update_resource_bundle_entry
( p_locale in varchar2
, p_context1 in varchar2 default null
, p_resource_key in varchar2
, p_resource_value in varchar2
)
is
pragma autonomous_transaction;
begin
  merge into resource_bundle_entries rby
  using (   select  p_locale     locale
            ,       p_context1   context1
            ,       p_resource_key  resource_key
            ,       p_resource_value resource_value
            from    dual
        ) nrb
  on ( rby.resource_key = nrb.resource_key
       and
       rby.locale = nrb.locale
       and
       ( (nrb.context1 is null and rby.context1 is null) 
          or
          nrb.context1 = rby.context1
       )
     )
  when matched
  then
    update
    set    resource_value = nrb.resource_value
    delete where resource_value is null
  when not matched
  then
    insert
    ( locale, context1, resource_key, resource_value)
    values
    ( nrb.locale, nrb.context1, nrb.resource_key, nrb.resource_value)    
  ;
  commit; -- commit only this data manipulation, inside the scope of an autonomous transactions
  
end update_resource_bundle_entry;


end nls_resource_manager;