package nl.amis.view;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.el.ValueExpression;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

import oracle.adf.model.BindingContext;
import oracle.adf.model.OperationBinding;
import oracle.adf.model.binding.DCBindingContainer;
import oracle.adf.view.rich.component.rich.nav.RichCommandImageLink;
import oracle.adf.view.rich.component.rich.output.RichContextInfo;
import oracle.adf.view.rich.event.DialogEvent;


public class TextEditor {

    // the UPDATE_RESOURCE_BUNDLE_ENTRY_BINDING is the id for the ADF Data Binding Control Binding through which the
    // resource bundle entries can be updated - for example a binding of the method
    // updateResourceBundleEntry(String locale, String context, String resourceKey, String resourceValue)
    // on an application module impl class
    private static final String UPDATE_RESOURCE_BUNDLE_ENTRY_BINDING =
        "updateResourceBundleEntry";
    private final String GENERIC_PAGEDEFINITION =
        "nl_amis_simple_view_GenericPageDef";

    // this el expression is evaluated to determine the context for the resource bundle entries
    // note: context is an additional dimension - next to language - that can be used (optionally) to
    // associated resource bundle entries which specific conditions or user groups
    private static final String EL_EXPRESSION_FOR_CONTEXT =
        "#{msgMgr.contextDimensionValue}";

    // EL expression used when referencing the message bundle
    private static final String EL_EXPRESSION_FOR_MSG_BUNDLE = "#{msg[";

    private Map<String, String> propertyLabels = new HashMap<String, String>();

    private List<TextResource> resources;


    public TextEditor() {
        propertyLabels.put("label", "Prompt");
        propertyLabels.put("shortDesc", "Hint Text");
        propertyLabels.put("text", "Text");
        propertyLabels.put("value", "Value");
    }

    public void handleDialogEvent(DialogEvent event) {
        if (event.getOutcome().equals(DialogEvent.Outcome.ok)) {
            recordResourceEntries();
        }
    }

    private void recordResourceEntries() {
        String context =
            (String)MessageManager.resolveExpression(EL_EXPRESSION_FOR_CONTEXT);
        Locale locale =
            FacesContext.getCurrentInstance().getViewRoot().getLocale();
        String language = locale.getLanguage();
        for (TextResource resource : this.getResources()) {
            //only when the text resource is changed should the update be invoked
            if (resource.isChanged()) {
                updateResourceBundleEntry(language, context, resource.getKey(),
                                          resource.getValue());
            }
        }
    }

    public void setPopuplauncher(UIComponent popuplauncher) {
        populateProperties(popuplauncher);
    }


    /**
     * this method retrieves the text resources associated with the UIComponent object and stores them
     * in the this.resources collection; subsequently, the resources are shown in the text editor popup.
     *
     **/
    private void populateProperties(UIComponent object) {
        //TODO handle messages with placeholders (:1 or {1})
        //TODO handle text resources associated with child elements (for example validator messages for inputText)
        //TODO also show non-text properties that are governed through a message bundle entry - such as format, styleClass, icon - check for #{msg[
        this.resources = new ArrayList<TextResource>();
        if (object instanceof RichCommandImageLink) {
            Object x = object.getParent().getChildren().get(0);
            if (x != object) {
                populateProperties(object.getParent().getChildren().get(0));
            }
        } else {
            if (object instanceof RichContextInfo) {
                populateProperties(object.getParent());
            }

            else {
                // generic code to extract all properties for all types of components
                // that refer to
                Iterator i = object.getAttributes().keySet().iterator();
                ValueExpression expression;
                while (i.hasNext()) {
                    Object o = i.next();
                    String propertyName = (String)o;
                    expression = object.getValueExpression(propertyName);
                    if (expression != null) {
                        System.out.println(propertyName + ": " +
                                           expression.getExpressionString());
                        if (expression.getExpressionString().indexOf(EL_EXPRESSION_FOR_MSG_BUNDLE) >
                            -1)
                            //TODO provide user friendly indications of properties (instead of the internal name)
                            extractTextResource(object,
                                                propertyLabels.containsKey(propertyName) ?
                                                propertyLabels.get(propertyName) :
                                                propertyName, propertyName);
                    }
                }//while
            }
        }
    }

    private void extractTextResource(UIComponent component,
                                     String propertyLabel,
                                     String propertyName) {
        ValueExpression expression;
        TextResource tr;
        expression = component.getValueExpression(propertyName);
        try {
            String el = expression.getExpressionString();
            tr =
 new TextResource(propertyLabel, (String)expression.getValue(FacesContext.getCurrentInstance().getELContext()),
                  el, extractKeyFromExpression(el), component);
            this.resources.add(tr);
        } catch (Exception e) {
        }
    }


    public void setResources(List<TextResource> resources) {
        this.resources = resources;
    }

    public List<TextResource> getResources() {
        return resources;
    }

    private String extractKeyFromExpression(String labelEL) {
        // #{msg['']}
        int pos1 = labelEL.indexOf("['");
        int pos2 = labelEL.indexOf("']");
        String key = labelEL.substring(pos1 + 2, pos2);
        return key;
    }


    private void updateResourceBundleEntry(String locale, String context,
                                           String key, String value) {
        BindingContext bindingContext = BindingContext.getCurrent();
        if (bindingContext == null) {
            throw new RuntimeException("No ADFM Binding Context found!!");
        }

        DCBindingContainer container =
            bindingContext.findBindingContainer(GENERIC_PAGEDEFINITION);

        OperationBinding updateResourceBundleEntryBinding =
            (OperationBinding)container.findCtrlBinding(UPDATE_RESOURCE_BUNDLE_ENTRY_BINDING);
        updateResourceBundleEntryBinding.getParamsMap().put("locale", locale);
        updateResourceBundleEntryBinding.getParamsMap().put("context",
                                                            context);
        updateResourceBundleEntryBinding.getParamsMap().put("resourceKey",
                                                            key);
        updateResourceBundleEntryBinding.getParamsMap().put("resourceValue",
                                                            value);
        updateResourceBundleEntryBinding.execute();
    }

}
